"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const codegen_1 = require("@azure-tools/codegen");
const expression_1 = require("../expression");
const case_1 = require("./case");
class SwitchStatement extends codegen_1.Initializer {
    constructor(expression, cases, objectInitializer) {
        super();
        this.expression = expression;
        this.caseStatements = new Array();
        this.apply(objectInitializer);
        this.add(cases);
    }
    get implementation() {
        return `
switch ( ${this.expression.value} )
{
${codegen_1.indent(this.caseStatements.map(each => each.implementation).join(codegen_1.EOL))}
}`;
    }
    add(cases) {
        if (typeof cases === 'function') {
            cases = cases();
        }
        if (cases instanceof case_1.CaseStatement) {
            this.caseStatements.push(cases);
        }
        else {
            for (const caseStatement of cases) {
                this.caseStatements.push(caseStatement);
            }
        }
    }
}
exports.SwitchStatement = SwitchStatement;
function Switch(expression, cases, objectInitializer) {
    return new SwitchStatement(expression_1.toExpression(expression), cases, objectInitializer);
}
exports.Switch = Switch;
//# sourceMappingURL=switch.js.map