"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const codegen_1 = require("@azure-tools/codegen");
const access_modifier_1 = require("./access-modifier");
const attribute_1 = require("./attribute");
const method_1 = require("./method");
const property_1 = require("./property");
const linq_1 = require("@azure-tools/linq");
class Type extends codegen_1.Initializer {
    constructor(namespace, name, objectIntializer) {
        super();
        this.namespace = namespace;
        this.name = name;
        this.description = '';
        this.methods = new Array();
        this.properties = new Array();
        this.genericParameters = new Array();
        this.interfaces = new Array();
        this.accessModifier = access_modifier_1.Access.Public;
        this.attributes = new Array();
        this.partial = false;
        this.apply(objectIntializer);
        this.filename = name;
    }
    get attributeDeclaration() {
        return linq_1.length(this.attributes) > 0 ? `${this.attributes.joinWith(each => `${each.value}`, codegen_1.EOL)}${codegen_1.EOL}` : '';
    }
    get fileName() {
        return this.filename;
    }
    set fileName(value) {
        this.filename = value;
    }
    toString() {
        return this.fullName;
    }
    get allProperties() {
        const result = new Array(...this.properties);
        for (const parent of this.interfaces) {
            result.push(...parent.allProperties);
        }
        return result;
    }
    get genericDeclaration() {
        return linq_1.length(this.genericParameters) === 0 ? '' : `<${this.genericParameters.join(',')}>`;
    }
    get fullName() {
        return `${this.namespace.fullName}.${this.name}${this.genericDeclaration}`;
    }
    get declaration() {
        throw new Error('Not Implemented: property \'declaration\' in Type subclass ');
    }
    get definition() {
        throw new Error('Not Implemented: property \'definition\' in Type subclass ');
    }
    validation(propertyName) {
        return '/*VALIDATION*/';
    }
    addProperty(property) {
        this.properties.push(property);
        return property;
    }
    addMethod(method) {
        this.methods.push(method);
        return method;
    }
    add(item) {
        if (item instanceof method_1.Method) {
            this.methods.push(item);
            return item;
        }
        if (item instanceof property_1.Property) {
            this.properties.push(item);
            return item;
        }
        if (item instanceof attribute_1.Attribute) {
            this.attributes.push(item);
            return item;
        }
        throw Error(`FATAL - UNABLE TO ADD UNKNOWN TYPE for '${JSON.stringify(item)}'`);
    }
}
exports.Type = Type;
//# sourceMappingURL=type.js.map